/*
 * network.c -dl
 * Networking routines
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <netpacket/packet.h>


#include "network.h"
#include "utility.h"
#include "protodefs.h"

/*
 * Returns raw PF_PACKET socket, requires user built link layer
*/
int open_raw_pf(int proto)
{
	int rsock = 0;
	
	if((rsock = socket(PF_PACKET, SOCK_DGRAM, proto)) == -1){
		perror("socket():");
		return -1;
	}
	
	return rsock;
}

/*
 * Returns raw PF_INET socket, kernel can handle link layer
 * sockaddr_in used to sendto as per 'man raw(7)' IP_HDRINCL 
 * implied by IPPROTO_RAW
*/
int open_raw_inet(int proto)
{
	int isock = 0;
	
	if((isock = socket(PF_INET, SOCK_RAW, proto)) == -1){
		perror("socket():");
		return -1;
	}
	
	return isock;
}

/* 
 * Bind Socket to Interface, allows for sniffing packets directly of the
 * device driver, returns 0 on success or -1 on failure
*/
int bind_rsock2dev(char *iface, int rsock, int proto)
{
	struct sockaddr_ll sll;
	struct ifreq ifr;
	
	memset(&ifr, 0, sizeof(ifr));
	strncpy(ifr.ifr_name, iface, IF_NAMESIZE);
	if((ioctl(rsock, SIOCGIFINDEX, &ifr)) == -1){
		perror("ioctl():");
		return -1;
	}

	memset(&sll, 0, sizeof(sll));
	sll.sll_family = AF_PACKET;
	sll.sll_ifindex = ifr.ifr_ifindex;

	if(bind(rsock, (struct sockaddr *)&sll, sizeof(sll)) == -1){
		perror("bind():");
		return -1;
	}
	
	return 0;
}

/*
 * Grabs local network interface information and stores in a ifcfg_t 
 * defined in network.h, returns 0 on success -1 on failure
*/
int get_local_info(int rsock, ifcfg_t *ifcfg)
{
	struct ifreq ifr;
	
	memset(&ifr, 0, sizeof(ifr));
	strncpy(ifr.ifr_name, ifcfg->iface, IF_NAMESIZE);
	if((ioctl(rsock, SIOCGIFHWADDR, &ifr)) == -1){
		perror("ioctl():");
		return -1;
	}
	memcpy(&(ifcfg->hwa), &ifr.ifr_hwaddr.sa_data, 6); //FIXME:
	
	memset(&ifr, 0, sizeof(ifr));
	strncpy(ifr.ifr_name, ifcfg->iface, IF_NAMESIZE);
	if((ioctl(rsock, SIOCGIFADDR, &ifr)) == -1){
		perror("ioctl():");
		return -1;
	}
	memcpy(&ifcfg->ipa, &(*(struct sockaddr_in *)&ifr.ifr_addr).sin_addr, 4);
	
	memset(&ifr, 0, sizeof(ifr));
	strncpy(ifr.ifr_name, ifcfg->iface, IF_NAMESIZE);
	if((ioctl(rsock, SIOCGIFBRDADDR, &ifr)) == -1){
		perror("ioctl():");
		return -1;
	}
	memcpy(&ifcfg->bcast, &(*(struct sockaddr_in *)&ifr.ifr_broadaddr).sin_addr, 4);
	
	memset(&ifr, 0, sizeof(ifr));
	strncpy(ifr.ifr_name, ifcfg->iface, IF_NAMESIZE);
	if((ioctl(rsock, SIOCGIFNETMASK, &ifr)) == -1){
		perror("ioctl():");
		return -1;
	}
	memcpy(&ifcfg->nmask.s_addr, &(*(struct sockaddr_in *)&ifr.ifr_netmask).sin_addr, 4);
	
	memset(&ifr, 0, sizeof(ifr));
	strncpy(ifr.ifr_name, ifcfg->iface, IF_NAMESIZE);
	if((ioctl(rsock, SIOCGIFMTU, &ifr)) == -1){
		perror("ioctl():");
		return -1;
	}
	ifcfg->mtu = ifr.ifr_mtu;
	
	return 0;
}

/*
 * Resolves hostname to IP returns 0 on success -1 on failure
*/
int resolve_name(char *host, struct in_addr *saddr)
{
	struct hostent *he;

	if(inet_aton(host, saddr) == 0){
		if((he = gethostbyname(host)) == NULL){
			herror("gethostbyname():");
			return -1;
		}
		memcpy(saddr, he->h_addr_list[0], 4);
	}
	
	return 0;
}

/*
 * Reverse lookup IP to name, returns 0 on success -1 on failure
*/
int resolve_ip(char *host, struct in_addr *saddr)
{
	struct hostent *he;

	if((he = gethostbyaddr(saddr, sizeof(saddr), AF_INET)) == 0){
		herror("gethostbyaddr()");
		return -1;
	}

	host = xrealloc(host, strlen(he->h_name) + 1);
	strncpy(host, he->h_name, strlen(he->h_name)); 
	
	return 0;
}

/*
 * Send packet on PF_INET socket
 *
*/
int send_raw_inet(int rsock, char *packet, int pksize, struct in_addr rhost)
{
	struct sockaddr_in dest;
	memset(&dest, 0, sizeof(struct sockaddr_in));

    dest.sin_family=PF_INET;
    dest.sin_port=htons(0);
    memcpy(&dest.sin_addr, &rhost , 4);

    if (sendto(rsock, packet, pksize, 0, (struct sockaddr *)&dest,
    			sizeof(struct sockaddr_in)) < 0) {
        perror("sendto()");
        return -1;
    }

    return 0;
}

/*
 * Receive data on PF_PACKET
*/
int recv_raw_pf(int rsock, char *packet, int pksize)
{
	struct sockaddr_ll from;
	socklen_t fromlen = sizeof(from);
	int bytes;
	
	memset(&from,0,sizeof(struct sockaddr_ll));
	if((bytes = recvfrom(rsock, packet, pksize, 0, (struct sockaddr*)&from,
			&fromlen)) == -1){
		return -1;
	}
	
	return bytes;
}

/*
 * Send packet using the PF_PACKET socket
*/
/* FIXME: Problem with dest mac, howto find?? using seperate PF_INET, SOCK_RAW 
          socket to send packets for now.
int send_dgram_pf(int rsock, char *packet, int plen, ifcfg_t *ifcfg)
{
	struct sockaddr_ll sll;
	struct ifreq ifr;
	sll = memset(&sll, 0, sizeof(struct sockaddr_ll));
	ifr = memset(&ifr, 0, sizeof(struct ifreq));
	
	strncpy(ifr.ifr_name, iface, IF_NAMESIZE);
	if((ioctl(rsock, SIOCGIFINDEX, &ifr)) < 0){
		perror("ioctl()");
		return -1;
	}
	else{ sll.sll_ifindex = ifr.ifr_ifindex; }
	
	memcpy(sll.sll_addr, ifcfg.hwa.sa_data, ETH_ALEN);
	sll.sll_halen = ETH_ALEN;
	sll.sll_family = AF_PACKET;
	sll.sll_protocol = htons(ETHERTYPE_IP);
	
	if((sendto(rsock, packet, plen, 0, (struct sockaddr *)&sll,
				sizeof(struct sockaddr_ll))) < 0){
		perror("sendto()");
		return -1;
	}
	
	return 0;
}
*/



