/*
 * packet_headers.c  -dl
 * Packet header construction routines
*/
#include <stdio.h>
#include <string.h>        // memcpy()
#include <stdlib.h>
#include <arpa/inet.h>     // htons() htonl()
#include <netinet/ether.h>

#include "utility.h"
#include "packet_headers.h"


u_short calc_csum(u_char *data, int len)
{
	u_long sum = 0;
	u_short *ip_head = (u_short *)data;

	while( len > 1 ){
		sum += *ip_head++;
		len -= 2;
	}

	/* Add leftover byte */
	if(len > 0)
		sum += (u_short) *(u_char *)ip_head;
	
	/* Make 16 bit */
	while( sum >> 16 )
		sum = (sum >> 16) + (sum & 0xffff);
		
	return ~sum; 
}

etherhdr_t *build_ether_hdr(linkcfg_t linkcfg)
{
	etherhdr_t *ether_head;
	/* allocate memory */
	ether_head = xmalloc(sizeof(etherhdr_t));

	/* Copy source and destination mac addresses into struct feilds */
	memcpy(ether_head->ether_shost, (void *)ether_aton(linkcfg.smac), ETH_ALEN);
	memcpy(ether_head->ether_dhost, (void *)ether_aton(linkcfg.dmac), ETH_ALEN);

	/* Copy over ETHERTYPE */
	ether_head->ether_type = htons(linkcfg.ethp);

	return (ether_head);
}

iphdr_t *build_ip_hdr(ipcfg_t ipcfg, int trans_sz, int payload_sz)
{
	iphdr_t *ip_hdr;
	
	ip_hdr = xmalloc(sizeof(iphdr_t));

	
	ip_hdr->ip_hl   = 5;
	ip_hdr->ip_v    = 4; 
	ip_hdr->ip_tos  = 0; 
	ip_hdr->ip_len  = htons(sizeof(iphdr_t) + trans_sz + payload_sz);
	ip_hdr->ip_id   = htons(ipcfg.ip_id); 
	ip_hdr->ip_off  = htons(ipcfg.frag_off);
	ip_hdr->ip_ttl  = ipcfg.ttl;
	ip_hdr->ip_p    = ipcfg.ipp;
	ip_hdr->ip_sum  = 0;
	memcpy(&ip_hdr->ip_src, &ipcfg.src, 4);
	memcpy(&ip_hdr->ip_dst, &ipcfg.dest, 4);
	
	ip_hdr->ip_sum = calc_csum((u_char *)ip_hdr, sizeof(iphdr_t));
	
	return(ip_hdr);
}

tcphdr_t *build_tcp_hdr(tcpcfg_t tcpcfg, iphdr_t *ip_hdr, char *payload, 
						int payload_sz)
{
	psuhdr_t *hdr;
	int size = 0;
	u_char *psu_tcp;
	
	tcphdr_t *tcp_hdr;
	tcp_hdr = xmalloc(sizeof(tcphdr_t));
	
	tcp_hdr->th_sport  = htons(tcpcfg.sport);
	tcp_hdr->th_dport  = htons(tcpcfg.dport);
	tcp_hdr->th_seq    = htonl(tcpcfg.seqn);
	tcp_hdr->th_ack  = htonl(tcpcfg.ackn);
	tcp_hdr->th_x2    = 0;
	tcp_hdr->th_off    = (sizeof(tcphdr_t) + tcpcfg.optsz) / 4;
	tcp_hdr->th_flags = tcpcfg.tflags;
	tcp_hdr->th_win  = htons(tcpcfg.winsz);
	tcp_hdr->th_sum	 = 0;   
	tcp_hdr->th_urp  = tcpcfg.urgp;

	/* Calculate the tcp checksum, this is done over the entire tcp hdr + */
    /* the data + a psuedo header	*/
	hdr = build_pseudo(ip_hdr);
	
	/* Allocate mem for header + pseudo + data */
	size = payload_sz + sizeof(tcphdr_t) + sizeof(psuhdr_t) + tcpcfg.optsz;
	psu_tcp = malloc(size);
	memset(psu_tcp, 0, size);
	
	/* Fill in length of header + data */
	hdr->len = htons(size - sizeof(psuhdr_t));
	
	/* Build data to be checksumed */	
	memcpy(psu_tcp, hdr, sizeof(psuhdr_t));
	memcpy(psu_tcp + sizeof(psuhdr_t), tcp_hdr, sizeof(tcphdr_t));
	memcpy(psu_tcp + sizeof(psuhdr_t) + sizeof(tcphdr_t), tcpcfg.tcpopts,
		tcpcfg.optsz);
	memcpy(psu_tcp + sizeof(psuhdr_t) + sizeof(tcphdr_t) + tcpcfg.optsz,
		payload, payload_sz);
	
	tcp_hdr->th_sum = calc_csum((u_char *)psu_tcp, size);
	
	free(psu_tcp);
	return(tcp_hdr);
}

psuhdr_t *build_pseudo(iphdr_t *ip_hdr)
{

	psuhdr_t *hdr;
	hdr = xmalloc(sizeof(psuhdr_t));
	
	hdr->src_ip  = ip_hdr->ip_src;
	hdr->dest_ip = ip_hdr->ip_dst;
	hdr->reserved = 0;
	hdr->protocol = ip_hdr->ip_p;
	hdr->len = 0;
	
	return(hdr);
}



